@@ -446,7 +446,7 @@ static inline void display_gbmodpoly_cf_qq(FILE *file,
     fprintf(file, ", \n");
   }
   display_modpoly(file, modgbs, p-1, gens);
-  fprintf(file, "\n");
+  /* fprintf(file, "\n"); */
   fprintf(file, "]:\n");
 }
 
@@ -466,7 +466,7 @@ static inline void display_lm_gbmodpoly_cf_qq(FILE *file,
     fprintf(file, ", \n");
   }
   display_monomial_single(file, gens, p-1, &modgbs->ldm);
-  fprintf(file, "\n");
+  /* fprintf(file, "\n"); */
   fprintf(file, "]:\n");
 }
 
@@ -499,18 +499,22 @@ static inline void gb_modpoly_clear(gb_modpoly_t modgbs){
       free(modgbs->modpolys[i]->cf_32[j]);
       mpz_clear(modgbs->modpolys[i]->cf_zz[j]);
     }
+
     for(uint32_t j = 0; j < 2 * modgbs->modpolys[i]->len; j++){
       mpz_clear(modgbs->modpolys[i]->cf_qq[j]);
     }
+
     mpz_clear(modgbs->modpolys[i]->lm);
     free(modgbs->modpolys[i]->cf_32);
     free(modgbs->modpolys[i]->cf_zz);
     free(modgbs->modpolys[i]->cf_qq);
+    free(modgbs->hm[i]);
   }
+
   free(modgbs->lmps);
   free(modgbs->hm);
-  //free_hash_table(&(modgbs->bht));
-  free_shared_hash_data(modgbs->bht);
+  free(modgbs->cf_64);
+  full_free_hash_table(&modgbs->bht);
   free(modgbs->modpolys);
 }
 
@@ -786,6 +790,7 @@ static int32_t gb_modular_trace_learning(gb_modpoly_t modgbs,
     else{
       gb_modpoly_init(modgbs, 2, lens, bs, bht->nv - st->nev, num_gb[0], leadmons[0], st);
     }
+    free(lens);
     modpgbs_set(modgbs, bs, bht, fc, start, st->nev);
     int is_empty = 0;
     if(bs->lml == 1){
@@ -1695,8 +1700,8 @@ uint64_t export_results_from_groebner_qq(
         len_t idx = gb->lmps[p];
         hm  = gb->hm[idx]+OFFSET;
         int32_t l = gb->modpolys[p]->len;
-        for(int32_t n = 0; n < nv; n++){
-            exp[term * nve + n + elim_block_len] = gb->ldm[p * nv + n];
+        for(int32_t n = 0; n < nve; n++){
+            exp[term * nv + n + elim_block_len] = gb->ldm[p * nv + n];
         }
         mpz_set(cf_qq[term], gb->modpolys[p]->lm);
 
@@ -1704,8 +1709,8 @@ uint64_t export_results_from_groebner_qq(
         for(int32_t i = l-1; i >= 0; i--){
             
             if(mpz_cmp_ui(gb->modpolys[p]->cf_qq[2*i], 0) != 0){
-                for(int32_t n = 0 ; n < nv; n++){
-                    exp[term * nve + n + elim_block_len] = ht->ev[hm[l-i]][evi[n]];
+                for(int32_t n = 0 ; n < nve; n++){
+                    exp[term * nv + n + elim_block_len] = ht->ev[hm[l-i]][evi[n]];
                 }
                 mpz_set(cf_qq[term], gb->modpolys[p]->cf_qq[2*i]);
 
@@ -1756,8 +1761,6 @@ gb_modpoly_t *groebner_qq(
   int32_t nr_vars = gens->nvars;
   int32_t nr_gens = gens->ngens;
   int reduce_gb = 1;
-  int32_t nr_nf = 0;
-  const uint32_t prime_start = pow(2, 30);
 
   /* timings */
   double ct0, ct1, rt0, rt1;
@@ -1778,7 +1781,7 @@ gb_modpoly_t *groebner_qq(
           nr_vars, nr_gens, 0 /* # normal forms */, ht_size,
           nr_threads, max_nr_pairs, reset_ht, la_option, use_signatures,
           reduce_gb, pbm_file, truncate_lifting, info_level);
- 
+
   /* all input generators are invalid */
   if (success == -1) {
       return NULL;
@@ -1804,11 +1807,12 @@ gb_modpoly_t *groebner_qq(
   md->f4_ctime = ct1 - ct0;
   md->f4_rtime = rt1 - rt0;
 
-  get_and_print_final_statistics(stderr, md, bs);
+  get_and_print_final_statistics(stdout, md, bs);
 
   /* free and clean up */
   free_mstrace(msd, md);
-
+  free_basis_without_hash_table(&bs);
+  free_trace(&md->tr);
   free(md);
   md    = NULL;
 
@@ -1971,18 +1975,21 @@ int64_t export_groebner_qq(
     md->f4_ctime = ct1 - ct0;
     md->f4_rtime = rt1 - rt0;
 
-    get_and_print_final_statistics(stderr, md, bs);
+    get_and_print_final_statistics(stdout, md, bs);
 
     /* free and clean up */
     free_mstrace(msd, md);
+    free_basis_without_hash_table(&bs);
+    free_trace(&md->tr);
     free(md);
     md    = NULL;
 
     int64_t nterms  = export_results_from_groebner_qq(bld, blen, bexp,
             bcf, mallocp, elim_block_len, (*modgbsp));
 
-    gb_modpoly_clear((*modgbsp));
-    
+    gb_modpoly_clear(*modgbsp);
+    free(modgbsp);
+
     return nterms;
 
 }
